/*
**	mod_snmp.c	- SNMP Interface module
**
**
** Copyright (c) 2001  Hughes Technologies Pty Ltd
**
** Permission to use, copy, and distribute for non-commercial purposes,
** is hereby granted without fee, providing that the above copyright
** notice appear in all copies and that both the copyright notice and this
** permission notice appear in supporting documentation.
**
** The software may be modified for your own purposes, but modified versions
** may not be distributed.
**
** This software is provided "as is" without any expressed or implied warranty.
**
*/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <netinet/in.h>
#include <netdb.h>

#include <ctype.h>
#include <sys/time.h>
#include <sys/types.h>

#include <ember.h>
#include <ember-config.h>

#include "nk_common.h"
#include "nk_snmp.h"
#include "mod_snmp.h"


/*
** Internal session handle structure
*/

typedef struct snmpfd_s {
	int	fd;
	char 	*session;
	struct snmpfd_s *next;
} snmpfd_t;



/*
** Globals
*/

static	snmpfd_t *snmpFD = NULL;
static	int	curFD = 1;





/****************************************************************************
** 	_snmpFindSession
**
**	Purpose	: Locate a session struct given a session handle
**	Args	: fd (session handle)
**	Returns	: pointer to session struct
**	Notes	: 
*/

static snmpfd_t *snmpFindSession(fd)
	int	fd;
{
	snmpfd_t *cur;

	cur = snmpFD;
	while(cur)
	{
		if (cur->fd == fd)
			break;
		cur = cur->next;
	}
	return(cur);
}





/****************************************************************************
** 	_snmpOpen
**
**	Purpose	: Open an SNMP session with a host
**	Args	: hostname and community
**	Returns	: Nothing
**	Notes	: 
*/

int doSnmpOpen(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*host,
		*community;
	snmpfd_t *new;
	char	*session;
	int	result;

	host = emTextArg(args);
	community = emTextArg(args->next);
	session = nkSnmpOpen(host, community);
	if (!session)
	{
		eDynamicSetError(script, EMBER_TYPE_INT, nkSnmpGetError());
		return(0);
	}

	curFD++;
	new = (snmpfd_t *)malloc(sizeof(snmpfd_t));
	if (!new)
	{
		eDynamicSetError(script, EMBER_TYPE_INT, "Out of memory");
		return(0);
	}
	result = new->fd = curFD;
	new->session = session;
	new->next = snmpFD;
	snmpFD = new;
	eDynamicSetIntValue(script, &script->returnValue, result);
	return(0);
}




/****************************************************************************
** 	_snmpClose
**
**	Purpose	: Shut down an snmp session
**	Args	: (int) session descriptor
**	Returns	: Nothing
**	Notes	: Releases the session descriptor and associated data
*/

int doSnmpClose(script, args)
	ember	*script;
	e_stack	*args;
{
	int	fd;
	snmpfd_t *cur,
		 *prev;

	fd = emIntArg(args);
	cur = snmpFD;
	prev = NULL;
	while(cur)
	{
		if (cur->fd == fd)
		{
			nkSnmpClose(cur->session);
			if (!prev)
				snmpFD = snmpFD->next;
			else
				prev->next = cur->next;
			free(cur);
			cur = NULL;
			return(0);
		}
		prev = cur;
		cur = cur->next;
	}
	if (!cur)
	{
		eDynamicSetError(script, EMBER_TYPE_INT, "Bad SNMP descriptor");
		return(0);
	}
	return(0);
}



/****************************************************************************
** 	_snmpop
**
**	Purpose	: GET/GETNEXT engine
**	Args	: operator (GET, GETNEXT), fd and variables
**	Returns	: Nothing
**	Notes	: 
*/

static int snmpop(script, args, op)
	ember	*script;
	e_stack	*args;
	int	op;
{
	char		*var,
			*val,
			*request,
			resBuf[10240];
	int		fd,
			numVars,
			count;
	snmpfd_t 	*sfd;
	e_value		*varArray,
			*valArray,
			*tmpPtr,
			tmpVal;
	nk_snmp_var	*curVar;
	nk_snmp_result	*result;

	fd = emIntArg(args);
	varArray = emArrayArg(args->next);
	if (op == NK_SNMP_SET)
		valArray = emArrayArg(args->next->next);
	else
		valArray = NULL;
	numVars = eDynamicGetNumArrayElements(script,varArray);

	request = nkSnmpCreatePDU(op);
	count = 0;
	while(count < numVars)
	{
		tmpPtr = eDynamicGetArrayElement(script,varArray,count);
		var = tmpPtr->value.textVal;
		if (valArray)
		{
			tmpPtr=eDynamicGetArrayElement(script,valArray,count);
			val = tmpPtr->value.textVal;
		}
		else
			val = NULL;
		if (nkSnmpAddVariable(request, var, val) < 0)
		{
			eDynamicSetError(script,EMBER_TYPE_TEXT, 
				nkSnmpGetError());
			return(0);
		}
		count++;
	}

	sfd = snmpFindSession(fd);
	if (!sfd)
	{
		eDynamicSetError(script,EMBER_TYPE_TEXT, "Bad SNMP descriptor");
                return(0);
	}

	result = nkSnmpRequest(sfd->session, request);
	if (result == NULL)
	{
		eDynamicSetError(script,EMBER_TYPE_TEXT, nkSnmpGetError());
		return(0);
	}

	count = 0;
	curVar = result->variables;
	while(curVar)
	{
		sprintf(resBuf,"%s%s%s",curVar->var, EMBER_BRK,
			curVar->text_val);
		eDynamicSetTextValue(script, &tmpVal, resBuf);
		eDynamicSetArrayElement(script, &script->returnValue,
			&tmpVal, count);
		curVar = curVar->next;
		count++;
	}
	return(0);
}


int doSnmpGet(script, args)
	ember	*script;
	e_stack	*args;
{
	return(snmpop(script,args,NK_SNMP_GET));
}


int doSnmpGetNext(script, args)
	ember	*script;
	e_stack	*args;
{
	return(snmpop(script,args,NK_SNMP_GETNEXT));
}



int doSnmpSet(script, args)
	ember	*script;
	e_stack	*args;
{
	return(snmpop(script,args,NK_SNMP_SET));
}


int init_mod(script, modPtr)
	ember	*script;
        void    *modPtr;
{
	if (modPtr)
	{
        	if (eDynamicAddModule(script, modPtr, snmp_externs) < 0)
        	{
                	return(-1);
        	}
	}
	return(0);
}

